from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from _pytest.config import Config
    from _pytest.python import Function


def config_to_xfaillist_path(config: Config) -> Path:
    return Path(config.rootpath, "xfails.list")


def item_to_test_id(item: Function) -> str:
    function = item.function
    return f"{function.__module__}.{item.name}"



from __future__ import annotations

import json
from codecs import open
from typing import TYPE_CHECKING

from . import config_to_xfaillist_path, item_to_test_id

if TYPE_CHECKING:
    from typing import Set

    from _pytest.config import Config
    from _pytest.python import Function
    from _pytest.runner import CallInfo

_faillist: Set[str] = set()


def pytest_runtest_makereport(item: Function, call: CallInfo[None]) -> None:
    if call.excinfo:
        _faillist.add(item_to_test_id(item))


def pytest_unconfigure(config: Config) -> None:
    path = config_to_xfaillist_path(config)
    with open(str(path), "w", encoding="UTF-8") as f:
        json.dump(sorted(list(_faillist)), f, indent=4, sort_keys=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_xfaillist']

package_data = \
{'': ['*']}

install_requires = \
['pytest>=6.2.2,<7.0.0']

entry_points = \
{'pytest11': ['pytest-xfaillist = pytest_xfaillist.hooks']}

setup_kwargs = {
    'name': 'pytest-xfaillist',
    'version': '0.1.1',
    'description': 'Maintain a xfaillist in an additional file to avoid merge-conflicts.',
    'long_description': 'pytest-xfaillist\n===============\n\nMaintain a xfaillist in an additional file to avoid merge-conflicts.\n\nDevelop\n=======\n\nWe have to workaround the missing support of "install -e ." in poetry. To\ndevelop please call:\n\n```bash\n./c/develop\n```\n',
    'author': 'Adfinis AG',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/projectcaluma/pytest-xfaillist',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

import json
from typing import TYPE_CHECKING

from _pytest.python import Function
from pytest import mark

from . import config_to_xfaillist_path, item_to_test_id

if TYPE_CHECKING:
    from typing import Set

    from _pytest.config import Config
    from _pytest.config.argparsing import Parser
    from _pytest.main import Session

_faillist: Set[str] = set()


def pytest_addoption(parser: Parser) -> None:
    parser.addoption(
        "--generate-xfaillist", action="store_true", help="Generate xfail.list"
    )


def pytest_configure(config: Config):
    if config.option.generate_xfaillist:
        config.pluginmanager.import_plugin("pytest_xfaillist.generate")
    else:
        path = config_to_xfaillist_path(config)
        if path.exists():
            with open(str(path), "r", encoding="UTF-8") as f:
                _faillist.update(set(json.load(f)))


def pytest_collection_finish(session: Session) -> None:
    for item in session.items:
        if isinstance(item, Function):
            id = item_to_test_id(item)
            if id in _faillist:
                item.add_marker(mark.xfail(reason="in xfail.list", strict=True))
